<?php
namespace Controller;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class TaskRelationship extends \Generated\Controller\TaskRelationship {
    /** @var \Service\TaskPrecursor $taskRelationshipService */
    protected $taskRelationshipService;

    public function __construct(Manager $dataSourceManager, \Service\TaskPrecursor $taskRelationshipService){
        $this->taskRelationshipService = $taskRelationshipService;
        parent::__construct($dataSourceManager);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response){
        parent::find($request, $response);
        $response->getDataSets($this->dataSourceName)[0]->setDataSourceName(explode('/',$request->getRoutePath())[1]);
    }

	public function save(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        /** @var DTO\Request\DataSet $request */
        $dataSet     = $request->getDataSet();
        $showMessage = $this->verifyShowMessage($dataSet);
        parent::save($request, $response, $showMessage);
        $rows = $dataSet->getRows();
        foreach ($rows as $row) {
            $taskRelationshipId = $row['TASK_RELATIONSHIP'];
            $type = $row['TYPE'];
            $originTaskId = $row['TASK'];
            if($taskRelationshipId && ($type == Entities\TaskRelationship::TEST_TYPE || $type == Entities\TaskRelationship::MERGE_REQUEST_TYPE)) {
                $this->taskRelationshipService->setDescriptionByTaskRelationship($taskRelationshipId, $type, $originTaskId);
            }
            $this->taskRelationshipService->saveFirstRelationshipById($taskRelationshipId, $originTaskId, $type);
            $this->taskRelationshipService->taskRelationshipWithGeneratingTaskIssue($originTaskId, $taskRelationshipId);

        }
        

    }

    public function delete(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $dataSet     = $request->getDataSet();
        $showMessage = $this->verifyShowMessage($dataSet);
        parent::delete($request, $response, $showMessage);
    }

    private function verifyShowMessage($dataSet) {      
        $showMessage = true;

        foreach ($dataSet->getRows() as $row) {
            if(isset($row['isMessage']) && $row['isMessage'] === false) {
                $showMessage = false;
                break;
            }
        }

        return $showMessage;
    }

    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function updateRelationshipMergeRequest(DTO\Request\DataSet $request, DTO\Response $response) {
        $task = $request->getDataSet()->getRows()[0];
        $taskId = $task['TASK'];
        try {
            $banchName = $this->taskRelationshipService->updateRelationshipMergeRequest($taskId);
            $response->addDataset(new DataSet('branchName', array($banchName)));
        } catch(\Exception $e){
            $response->addMessage(new DTO\Response\Message($e->getMessage()));
        }
    }
    
}