<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;

use Zeedhi\DevOps\Util\Utilities;

class Label extends \Doctrine\ORM\EntityRepository {

    public function getLabelsByUserAndProject($userId, $conditions = ""){
        $query = <<<SQL
        SELECT *
         FROM (
SELECT L.ID, L.RESOURCE_T, L.PROJECT, L.DESCRIPTION, P.NAME AS PROJECT_NAME, RP.RESOURCE_C, RP.TYPE, PT.LABEL
  FROM LABEL L
  LEFT JOIN PROJECT P
         ON L.PROJECT = P.ID
  LEFT JOIN RESOURCE_PROJECT RP
         ON RP.PROJECT = P.ID
        AND RP.TYPE   = 'M'
        AND RP.RESOURCE_C = :RESOURCE_T
  LEFT JOIN PROJECT_TYPE PT
         ON PT.ID = P.PROJECT_TYPE
 WHERE L.RESOURCE_T = :RESOURCE_T
    OR L.PROJECT IN (SELECT P1.ID
                       FROM PROJECT P1, RESOURCE_PROJECT RP
                      WHERE P1.STATUS <> 'D'
                        AND P1.PROJECT_STATUS IN (SELECT TS.ID
                                                    FROM TASK_STATUS TS
                                                   WHERE TS.IS_PROJECT = 'Y'
                                                     AND TS.CONTROL NOT IN ('C', 'X'))

                        AND P1.PROJECT_TYPE IN (SELECT PT.ID
                                                 FROM PROJECT_TYPE PT
                                                WHERE PT.EXTERNALTYPE IN ('P','M'))
                        AND RP.PROJECT = P1.ID
                        AND RP.RESOURCE_C = :RESOURCE_T
                        AND RP.TYPE = 'M'
                      UNION
                     SELECT P.ID
                       FROM PROJECT P
                      WHERE P.STATUS <> 'D'
                        AND P.PROJECT_TYPE IN (SELECT PT.ID
                                                 FROM PROJECT_TYPE PT
                                                WHERE PT.EXTERNALTYPE IN ('P'))
                        AND P.PROJECT_STATUS IN (SELECT TS.ID
                                                   FROM TASK_STATUS TS
                                                  WHERE TS.IS_PROJECT = 'Y'
                                                    AND TS.CONTROL NOT IN ('C', 'X'))
                        AND P.ID IN (SELECT PJ.ID
                                       FROM PROJECT PJ
                                      WHERE PJ.STATUS <> 'D'
                                        AND PJ.STRUCTURE IN (SELECT S.ID
                                                               FROM STRUCTURE S
                                                              WHERE S.ID IN (SELECT ST.ID
                                                                               FROM STRUCTURE ST, PERMISSION PE
                                                                              WHERE ST.ID = PE.STRUCTURE
                                                                                AND ST.STATUS <> 'D'
                                                                                AND PE.TYPE = 'M'
                                                                                AND PE.RESOURCE_T = :RESOURCE_T)
                                                                AND S.STATUS <> 'D'
                                                              UNION
                                                             SELECT S.ID
                                                               FROM STRUCTURE S
                                                              WHERE S.STATUS <> 'D'
                                                              START WITH S.PARENT IN (SELECT ST.ID
                                                                                        FROM STRUCTURE ST, PERMISSION PE
                                                                                       WHERE ST.ID = PE.STRUCTURE
                                                                                         AND ST.STATUS <> 'D'
                                                                                         AND PE.TYPE = 'M'
                                                                                         AND PE.RESOURCE_T = :RESOURCE_T)
                                                            CONNECT BY PRIOR S.ID = S.PARENT))
                      UNION
                     SELECT P.ID
                       FROM PROJECT P
                      WHERE P.STATUS <> 'D'
                        AND P.PROJECT_TYPE IN (SELECT PT.ID
                                                 FROM PROJECT_TYPE PT
                                                WHERE PT.EXTERNALTYPE IN ('P'))
                        AND P.PROJECT_STATUS IN (SELECT TS.ID
                                                   FROM TASK_STATUS TS
                                                  WHERE TS.IS_PROJECT = 'Y'
                                                    AND TS.CONTROL NOT IN ('C', 'X'))
                      START WITH P.PARENT IN (SELECT S.ID
                                                FROM STRUCTURE S
                                               WHERE S.ID IN (SELECT ST.ID
                                                                FROM STRUCTURE ST, PERMISSION PE
                                                               WHERE ST.ID = PE.STRUCTURE
                                                                 AND ST.STATUS <> 'D'
                                                                 AND PE.TYPE = 'M'
                                                                 AND PE.RESOURCE_T = :RESOURCE_T)
                                                 AND S.STATUS <> 'D'
                                               UNION
                                              SELECT S.ID
                                                FROM STRUCTURE S
                                               WHERE S.STATUS <> 'D'
                                               START WITH S.PARENT IN (SELECT ST.ID
                                                                         FROM STRUCTURE ST, PERMISSION PE
                                                                        WHERE ST.ID = PE.STRUCTURE
                                                                          AND ST.STATUS <> 'D'
                                                                          AND PE.TYPE = 'M'
                                                                          AND PE.RESOURCE_T = :RESOURCE_T)
                                             CONNECT BY PRIOR S.ID = S.PARENT)
                    CONNECT BY PRIOR P.ID = P.PARENT)
            ORDER BY L.DESCRIPTION)
        WHERE 1=1
              $conditions
SQL;

        $param = array(
            'RESOURCE_T' => $userId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }


    public function getLabelsByOperatorId($id,$projectIds = array(),$filter = '')
    {
        $projectIdString = Utilities::getStringForArray($projectIds,',','OR LAB.PROJECT IN(',')');

        $query = <<<SQL
        SELECT * FROM (
                SELECT LAB.*,PRO.ID AS PROJECT_ID,PRO.NAME AS PROJECT_NAME,PRT.ID AS PROJECT_TYPE_ID,PRT.LABEL AS PROJECT_TYPE_LABEL

                FROM LABEL LAB
                    LEFT JOIN PROJECT PRO ON (PRO.ID = LAB.PROJECT)
                    LEFT JOIN PROJECT_TYPE PRT ON(PRT.ID = PRO.PROJECT_TYPE)
                WHERE     LAB.RESOURCE_T = :id
                $projectIdString
                ORDER BY LAB.DESCRIPTION)
		WHERE 1 = 1
		 $filter

SQL;

        $param = array(
            'id' => $id
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        return $result;
    }

    public function findLabelsByTask($scheduleId,$typeTaskLabel)
    {

        $query = <<<SQL
               SELECT LAB.*,PRO.ID AS PROJECT_ID,PRO.NAME AS PROJECT_NAME,PRT.ID AS PROJECT_TYPE_ID,
                      PRT.LABEL AS PROJECT_TYPE_LABEL
		         FROM LABEL LAB
			     JOIN TASK_LABEL TAL ON(TAL.TASK = $scheduleId AND TAL.LABEL = LAB.ID AND TAL.TYPE = '$typeTaskLabel')
            LEFT JOIN PROJECT PRO ON (PRO.ID = LAB.PROJECT)
			LEFT JOIN PROJECT_TYPE PRT ON(PRT.ID = PRO.PROJECT_TYPE)
			 ORDER BY LAB.DESCRIPTION

SQL;

        $result = $this->getEntityManager()->getConnection()->fetchAll($query);
        return $result;
    }

    public function getLabelsByIssueId($issueId)
    {
        $query = <<<SQL
        SELECT            L.ID, L.DESCRIPTION
        FROM              TASK_LABEL TL
        LEFT JOIN         LABEL L ON (L.ID = TL.LABEL)
        WHERE             TL.TASK = :issueId
SQL;
        $param = array(
            'issueId' => $issueId
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        return $result;
    }

    public function findMeetingLabels($filters)
    {
        $filterString = Utilities::createQueryFiltered($filters);
        $query = <<<SQL
         SELECT *
            FROM(SELECT LAB.ID,LAB.DESCRIPTION,LAB.PROJECT
                  FROM LABEL LAB)
            WHERE 1 = 1
            $filterString
            ORDER BY DESCRIPTION
SQL;

        $result = $this->getEntityManager()->getConnection()->fetchAll($query);
        return $result;
    }
    public function findMeetingLabelsForSchedule($projectId)
    {


        $query = <<<SQL
           SELECT LAB.ID,LAB.DESCRIPTION,LAB.PROJECT
             FROM LABEL LAB
        LEFT JOIN PROJECT PRO ON(PRO.ID = :projectId)
            WHERE LAB.PROJECT = :projectId
               OR PRO.PARENT = LAB.PROJECT
         ORDER BY DESCRIPTION
SQL;

        $param = array(
            'projectId' => $projectId
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        return $result;
    }
}

